@@ -1,3 +1,26 @@
+/*
+Copyright (C) 2004-2008 Christian Wieninger
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 2
+of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+Or, point your browser to http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+
+The author can be reached at cwieninger@gmx.de
+
+The project's page is at http://winni.vdr-developer.org/epgsearch
+*/
+
 #include <vector>
 #include "epgsearchtools.h"
 #include "blacklist.h"
@@ -48,13 +71,14 @@ cBlacklist::cBlacklist(void)
     catvalues = (char**) malloc(SearchExtCats.Count() * sizeof(char*));
     cSearchExtCat *SearchExtCat = SearchExtCats.First();
     int index = 0;
-    while (SearchExtCat)
+    while (SearchExtCat) 
     {
 	catvalues[index] = (char*)malloc(MaxFileName);
 	*catvalues[index] = 0;
 	SearchExtCat = SearchExtCats.Next(SearchExtCat);
 	index++;
     }
+    ignoreMissingEPGCats = 0;
     fuzzyTolerance = 1;
 }
 
@@ -68,7 +92,7 @@ cBlacklist::~cBlacklist(void)
   {
       cSearchExtCat *SearchExtCat = SearchExtCats.First();
       int index = 0;
-      while (SearchExtCat)
+      while (SearchExtCat && index < (int)(sizeof(catvalues)/sizeof(char*))) 
       {
 	  free(catvalues[index]);
 	  SearchExtCat = SearchExtCats.Next(SearchExtCat);
@@ -85,7 +109,7 @@ cBlacklist& cBlacklist::operator= (const cBlacklist &Blacklist)
   catvalues = (char**) malloc(SearchExtCats.Count() * sizeof(char*));
   cSearchExtCat *SearchExtCat = SearchExtCats.First();
   int index = 0;
-  while (SearchExtCat)
+  while (SearchExtCat) 
   {
       catvalues[index] = (char*)malloc(MaxFileName);
       *catvalues[index] = 0;
@@ -117,7 +141,7 @@ void cBlacklist::CopyFromTemplate(const cSearchExt* templ)
 
   cSearchExtCat *SearchExtCat = SearchExtCats.First();
   int index = 0;
-  while (SearchExtCat)
+  while (SearchExtCat) 
   {
       strcpy(catvalues[index], templ->catvalues[index]);
       SearchExtCat = SearchExtCats.Next(SearchExtCat);
@@ -131,6 +155,7 @@ void cBlacklist::CopyFromTemplate(const cSearchExt* templ)
   if (templ->channelGroup)
       channelGroup = strdup(templ->channelGroup);
   fuzzyTolerance = templ->fuzzyTolerance;
+  ignoreMissingEPGCats = templ->ignoreMissingEPGCats;
 }
 
 bool cBlacklist::operator< (const cListObject &ListObject)
@@ -150,14 +175,10 @@ const char *cBlacklist::ToText(void)
   char* tmp_catvalues = NULL;
 
   free(buffer);
-  if (mode < 4 || mode == 5) // not a regular expression
-      asprintf(&tmp_search, "%s", search);
-  else if (mode ==4)
-  {
-      tmp_search = strdup(search);
-      while(strstr(tmp_search, "|"))
-	  tmp_search = strreplace(tmp_search, "|", "!^pipe^!"); // ugly: replace with something, that should not happen to be part of a regular expression
-  }
+  tmp_search = strdup(search);
+  while(strstr(tmp_search, "|"))
+    tmp_search = strreplace(tmp_search, "|", "!^pipe^!"); // ugly: replace a pipe with something, that should not happen to be part of a regular expression 
+
   strreplace(tmp_search, ':', '|');
 
   if (useTime)
@@ -174,9 +195,9 @@ const char *cBlacklist::ToText(void)
   if (useChannel==1)
   {
       if (channelMin->Number() < channelMax->Number())
-	  asprintf(&tmp_chanSel, "%s|%s", CHANNELSTRING(channelMin), CHANNELSTRING(channelMax));
+	  msprintf(&tmp_chanSel, "%s|%s", CHANNELSTRING(channelMin), CHANNELSTRING(channelMax));
       else
-	  asprintf(&tmp_chanSel, "%s", CHANNELSTRING(channelMin));
+	  msprintf(&tmp_chanSel, "%s", CHANNELSTRING(channelMin));
   }
   if (useChannel==2)
   {
@@ -197,18 +218,18 @@ const char *cBlacklist::ToText(void)
       while (SearchExtCat)
       {
 	  char* catvalue = NULL;
-	  asprintf(&catvalue, "%s", catvalues[index]);
+	  msprintf(&catvalue, "%s", catvalues[index]);
 	  while(strstr(catvalue, ":"))
 	      catvalue = strreplace(catvalue, ":", "!^colon^!"); // ugly: replace with something, that should not happen to be part ofa category value
 	  while(strstr(catvalue, "|"))
 	      catvalue = strreplace(catvalue, "|", "!^pipe^!"); // ugly: replace with something, that should not happen to be part of a regular expression
 
 	  if (index == 0)
-	      asprintf(&tmp_catvalues, "%d#%s", SearchExtCat->id, catvalue);
+	      msprintf(&tmp_catvalues, "%d#%s", SearchExtCat->id, catvalue);
 	  else
 	  {
 	      char* temp = tmp_catvalues;
-	      asprintf(&tmp_catvalues, "%s|%d#%s", tmp_catvalues, SearchExtCat->id, catvalue);
+	      msprintf(&tmp_catvalues, "%s|%d#%s", tmp_catvalues, SearchExtCat->id, catvalue);
 	      free(temp);
 	  }
 	  SearchExtCat = SearchExtCats.Next(SearchExtCat);
@@ -217,7 +238,7 @@ const char *cBlacklist::ToText(void)
       }
   }
 
-  asprintf(&buffer, "%d:%s:%d:%s:%s:%d:%s:%d:%d:%d:%d:%d:%d:%s:%s:%d:%d:%d:%s:%d",
+  msprintf(&buffer, "%d:%s:%d:%s:%s:%d:%s:%d:%d:%d:%d:%d:%d:%s:%s:%d:%d:%d:%s:%d:%d",
 	   ID,
 	   tmp_search,
 	   useTime,
@@ -237,7 +258,8 @@ const char *cBlacklist::ToText(void)
 	   DayOfWeek,
 	   useExtEPGInfo,
 	   useExtEPGInfo?tmp_catvalues:"",
-	   fuzzyTolerance);
+	   fuzzyTolerance,
+	   ignoreMissingEPGCats);
 
 
   if (tmp_chanSel)
@@ -360,6 +382,8 @@ bool cBlacklist::Parse(const char *s)
 		break;
 	    case 20: fuzzyTolerance = atoi(value);
 		break;
+            case 21: ignoreMissingEPGCats = atoi(value);
+         	break;
         } //switch
       }
       parameter++;
@@ -525,7 +549,7 @@ cEvent * cBlacklist::GetEventByBlacklist(const cSchedule *schedules, const cEven
      if (!p->Title() || strlen(p->Title()) == 0)
 	 continue;
 
-     asprintf(&szTest, "%s%s%s%s%s", (useTitle?p->Title():""), (useSubtitle||useDescription)?"~":"",
+     msprintf(&szTest, "%s%s%s%s%s", (useTitle?p->Title():""), (useSubtitle||useDescription)?"~":"",
 	      (useSubtitle?p->ShortText():""),useDescription?"~":"",
 	      (useDescription?p->Description():""));
 
