@@ -1,8 +1,29 @@
+/*
+Copyright (C) 2004-2008 Christian Wieninger
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 2
+of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+Or, point your browser to http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+
+The author can be reached at cwieninger@gmx.de
+
+The project's page is at http://winni.vdr-developer.org/epgsearch
+*/
+
 /*
  * epgsearch.c: A plugin for the Video Disk Recorder
  *
- * See the README file for copyright information and how to reach the author.
- *
  */
 
 #include <vector>
@@ -26,6 +47,7 @@
 #include "rcfile.h"
 #include "changrp.h"
 #include "recstatus.h"
+#include "timerstatus.h"
 #include "recdone.h"
 #include "blacklist.h"
 #include "menu_search.h"
@@ -42,14 +64,16 @@
 #include "epgsearchservices.h"
 #include "menu_quicksearch.h"
 #include "menu_announcelist.h"
+#include "confdloader.h"
 
-static const char VERSION[]        = "0.9.22";
-static const char DESCRIPTION[]    = "search the EPG for repeats and more";
+static const char VERSION[]        = "0.9.24";
+static const char DESCRIPTION[]    =  trNOOP("search the EPG for repeats and more");
 
 // globals
 char *ConfigDir = NULL;
 bool reloadMenuConf = false;
 int updateForced = 0;
+bool isUTF8 = false;
 
 // LogFile declaration and statics
 cLogFile LogFile;
@@ -64,11 +88,13 @@ cPluginEpgsearch::cPluginEpgsearch(void)
    showConflicts = false;
    showAnnounces = false;
    gl_recStatusMonitor = NULL;
+   gl_timerStatusMonitor = NULL;
 }
 
 cPluginEpgsearch::~cPluginEpgsearch()
 {
    delete gl_recStatusMonitor;
+   delete gl_timerStatusMonitor;
    cSearchTimerThread::Exit();
    cSwitchTimerThread::Exit();
    cConflictCheckThread::Exit();
@@ -377,13 +403,22 @@ bool cPluginEpgsearch::Service(const char *Id, void *Data)
       serviceData->handler = autoHandler;
       return true;
    }
-
+   if (strcmp(Id, "Epgsearch-services-v1.1") == 0) {
+      if (Data == NULL)
+         return true;
+      Epgsearch_services_v1_1* serviceData = (Epgsearch_services_v1_1*) Data;
+      std::auto_ptr<cEpgsearchServiceHandler> autoHandler(new cEpgsearchServiceHandler);
+      serviceData->handler = autoHandler;
+      return true;
+   }
    return false;
 }
 
 bool cPluginEpgsearch::Initialize(void)
 {
+#if APIVERSNUM < 10507
    RegisterI18n(Phrases);
+#endif
    return true;
 }
 
@@ -400,8 +435,14 @@ bool cPluginEpgsearch::Start(void)
       LogFile.Open(AddDirectory(CONFIGDIR, "epgsearch.log"), Version());
 
    gl_recStatusMonitor = new cRecStatusMonitor;
+   gl_timerStatusMonitor = new cTimerStatusMonitor;
 
    SearchExtCats.Load(AddDirectory(CONFIGDIR, "epgsearchcats.conf"), true);
+
+   LoadMenuTemplates();
+   LoadUserVars();
+   LoadConfD();
+
    ChannelGroups.Load(AddDirectory(CONFIGDIR, "epgsearchchangrps.conf"), true);
    Blacklists.Load(AddDirectory(CONFIGDIR, "epgsearchblacklists.conf"));
    SearchExts.Load(AddDirectory(CONFIGDIR, "epgsearch.conf"));
@@ -412,16 +453,36 @@ bool cPluginEpgsearch::Start(void)
    DefTimerCheckModes.Load(AddDirectory(CONFIGDIR, "deftimerchkmodes.conf"));
    TimersDone.Load(AddDirectory(CONFIGDIR, "timersdone.conf"));
 
-   LoadMenuTemplates();
-   LoadUserVars();
-
    cSearchTimerThread::Init(this);    
    cSwitchTimerThread::Init();    
    cConflictCheckThread::Init(this);    
 
+   CheckUTF8();
+
    return true;
 }
 
+void cPluginEpgsearch::CheckUTF8()
+{
+#if APIVERSNUM >= 10503 
+   // Taken from VDR's vdr.c
+   char *CodeSet = NULL;
+   if (setlocale(LC_CTYPE, ""))
+     CodeSet = nl_langinfo(CODESET);
+   else {
+     char *LangEnv = getenv("LANG"); // last resort in case locale stuff isn't installed
+     if (LangEnv) {
+       CodeSet = strchr(LangEnv, '.');
+       if (CodeSet)
+	 CodeSet++; // skip the dot
+     }
+   }
+   
+   if (CodeSet && strcasestr(CodeSet, "UTF-8") != 0)
+     isUTF8=true;
+#endif
+}
+
 void cPluginEpgsearch::Stop(void)
 {
    cSearchTimerThread::Exit();
@@ -450,6 +511,7 @@ cOsdObject *cPluginEpgsearch::DoInitialSearch(char* rcFilename)
       SearchExt->useTitle = rcFile.UseTitle;
       SearchExt->useSubtitle = rcFile.UseSubtitle;
       SearchExt->useDescription = rcFile.UseDescr;
+
       remove(rcFilename);
       return new cMenuSearchResultsForSearch(SearchExt, cTemplFile::GetTemplateByName("MenuSearchResults"));
    }
@@ -521,6 +583,12 @@ void cPluginEpgsearch::LoadUserVars()
    UserVars.InitExtEPGVars();
 }
 
+void cPluginEpgsearch::LoadConfD()
+{
+  cConfDLoader D;
+  D.Load();
+}
+
 cMenuSetupPage *cPluginEpgsearch::SetupMenu(void)
 {
    // Return a setup menu in case the plugin supports one.
