@@ -1,3 +1,26 @@
+/*
+Copyright (C) 2004-2008 Christian Wieninger
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 2
+of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+Or, point your browser to http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+
+The author can be reached at cwieninger@gmx.de
+
+The project's page is at http://winni.vdr-developer.org/epgsearch
+*/
+
 #include <vector>
 #include "epgsearchext.h"
 #include "epgsearchcfg.h"
@@ -49,7 +72,7 @@ cSearchExt::cSearchExt(void)
    useDescription = true;
    useDuration = false;
    minDuration = 0;
-   maxDuration = 130;
+   maxDuration = 2359;
    useAsSearchTimer = false;
    useDayOfWeek = false;
    DayOfWeek = 0;
@@ -89,6 +112,13 @@ cSearchExt::cSearchExt(void)
    fuzzyTolerance = 1;
    useInFavorites = 0;
    menuTemplate = 0;
+   delMode = 0;
+   delAfterCountRecs = 0;
+   delAfterDaysOfFirstRec = 0;
+   useAsSearchTimerFrom = 0;
+   useAsSearchTimerTil = 0;
+   ignoreMissingEPGCats = 0;
+   unmuteSoundOnSwitch = 0;
 }
 
 cSearchExt::~cSearchExt(void)
@@ -102,7 +132,7 @@ cSearchExt::~cSearchExt(void)
    {
       cSearchExtCat *SearchExtCat = SearchExtCats.First();
       int index = 0;
-      while (SearchExtCat) 
+      while (SearchExtCat && index < (int)(sizeof(catvalues)/sizeof(char*))) 
       {
          free(catvalues[index]);
          SearchExtCat = SearchExtCats.Next(SearchExtCat);
@@ -201,6 +231,13 @@ void cSearchExt::CopyFromTemplate(const cSearchExt* templ)
    fuzzyTolerance = templ->fuzzyTolerance;
    useInFavorites = templ->useInFavorites;
    menuTemplate = templ->menuTemplate;
+   delMode = templ->delMode;
+   delAfterCountRecs = templ->delAfterCountRecs;
+   delAfterDaysOfFirstRec = templ->delAfterDaysOfFirstRec;
+   useAsSearchTimerFrom = templ->useAsSearchTimerFrom;
+   useAsSearchTimerTil = templ->useAsSearchTimerTil;
+   ignoreMissingEPGCats = templ->ignoreMissingEPGCats;
+   unmuteSoundOnSwitch = templ->unmuteSoundOnSwitch;
 }
 
 bool cSearchExt::operator< (const cListObject &ListObject)
@@ -215,7 +252,7 @@ const char *cSearchExt::ToText()
    char tmp_Stop[5] = "";
    char tmp_minDuration[5] = "";
    char tmp_maxDuration[5] = "";
-   char* tmp_chanSel = NULL;
+   cString tmp_chanSel;
    char* tmp_search = NULL;
    char* tmp_directory = NULL;
    char* tmp_catvalues = NULL;
@@ -248,9 +285,9 @@ const char *cSearchExt::ToText()
    if (useChannel==1)
    {
       if (channelMin->Number() < channelMax->Number())
-         asprintf(&tmp_chanSel, "%s|%s", CHANNELSTRING(channelMin), CHANNELSTRING(channelMax));
+	tmp_chanSel = cString::sprintf("%s|%s", CHANNELSTRING(channelMin), CHANNELSTRING(channelMax));
       else
-         asprintf(&tmp_chanSel, "%s", CHANNELSTRING(channelMin));
+	tmp_chanSel = cString(CHANNELSTRING(channelMin));
    }
    if (useChannel==2)
    {
@@ -261,7 +298,7 @@ const char *cSearchExt::ToText()
          useChannel = 0;
       }
       else
-         tmp_chanSel = strdup(channelGroup);
+	tmp_chanSel = cString(channelGroup);
    }
 
    if (useExtEPGInfo)
@@ -271,18 +308,18 @@ const char *cSearchExt::ToText()
       while (SearchExtCat) 
       {
          char* catvalue = NULL;
-         asprintf(&catvalue, "%s", catvalues[index]);
+         msprintf(&catvalue, "%s", catvalues[index]);
          while(strstr(catvalue, ":"))
             catvalue = strreplace(catvalue, ":", "!^colon^!"); // ugly: replace with something, that should not happen to be part ofa category value
          while(strstr(catvalue, "|"))
             catvalue = strreplace(catvalue, "|", "!^pipe^!"); // ugly: replace with something, that should not happen to be part of a regular expression 
 
          if (index == 0)
-            asprintf(&tmp_catvalues, "%d#%s", SearchExtCat->id, catvalue);
+            msprintf(&tmp_catvalues, "%d#%s", SearchExtCat->id, catvalue);
          else
          {
             char* temp = tmp_catvalues;	      
-            asprintf(&tmp_catvalues, "%s|%d#%s", tmp_catvalues, SearchExtCat->id, catvalue);
+            msprintf(&tmp_catvalues, "%s|%d#%s", tmp_catvalues, SearchExtCat->id, catvalue);
             free(temp);
          }
          SearchExtCat = SearchExtCats.Next(SearchExtCat);	 
@@ -298,11 +335,11 @@ const char *cSearchExt::ToText()
       while (blacklistObj) 
       {
          if (index == 0)
-            asprintf(&tmp_blacklists, "%d", blacklistObj->blacklist->ID);
+            msprintf(&tmp_blacklists, "%d", blacklistObj->blacklist->ID);
          else
          {
             char* temp = tmp_blacklists;	      
-            asprintf(&tmp_blacklists, "%s|%d", tmp_blacklists, blacklistObj->blacklist->ID);
+            msprintf(&tmp_blacklists, "%s|%d", tmp_blacklists, blacklistObj->blacklist->ID);
             free(temp);
          }
          blacklistObj = blacklists.Next(blacklistObj);	 
@@ -310,14 +347,14 @@ const char *cSearchExt::ToText()
       }      
    }
 
-   asprintf(&buffer, "%d:%s:%d:%s:%s:%d:%s:%d:%d:%d:%d:%d:%d:%s:%s:%d:%d:%d:%d:%s:%d:%d:%d:%d:%d:%d:%d:%s:%d:%d:%d:%d:%d:%ld:%d:%d:%d:%d:%d:%d:%s:%d:%d:%d",
+   msprintf(&buffer, "%d:%s:%d:%s:%s:%d:%s:%d:%d:%d:%d:%d:%d:%s:%s:%d:%d:%d:%d:%s:%d:%d:%d:%d:%d:%d:%d:%s:%d:%d:%d:%d:%d:%ld:%d:%d:%d:%d:%d:%d:%s:%d:%d:%d:%d:%d:%d:%ld:%ld:%d:%d",
             ID,
             tmp_search,
             useTime,
             tmp_Start,
             tmp_Stop,
             useChannel,
-            (useChannel>0 && useChannel<3)?tmp_chanSel:"0",
+            (useChannel>0 && useChannel<3)?*tmp_chanSel:"0",
             useCase,
             mode,
             useTitle,
@@ -354,9 +391,15 @@ const char *cSearchExt::ToText()
             blacklists.Count()>0?tmp_blacklists:"",
             fuzzyTolerance,
             useInFavorites,
-            menuTemplate);
+            menuTemplate,
+            delMode,
+            delAfterCountRecs,
+            delAfterDaysOfFirstRec,
+	    useAsSearchTimerFrom,
+	    useAsSearchTimerTil,
+	    ignoreMissingEPGCats, 
+	    unmuteSoundOnSwitch);
 
-   if (tmp_chanSel) free(tmp_chanSel);
    if (tmp_search) free(tmp_search);
    if (tmp_directory) free(tmp_directory);
    if (tmp_catvalues) free(tmp_catvalues);
@@ -536,6 +579,24 @@ bool cSearchExt::Parse(const char *s)
                   break;
                case 44: menuTemplate = atoi(value);
                   break;
+               case 45: delMode = atoi(value);
+                  break;
+               case 46: delAfterCountRecs = atoi(value);
+                  break;
+               case 47: delAfterDaysOfFirstRec = atoi(value);
+                  break;
+	    case 48:
+	      useAsSearchTimerFrom = atol(value);
+	      break;
+	    case 49:
+	      useAsSearchTimerTil = atol(value);
+	      break;
+	    case 50:
+	      ignoreMissingEPGCats = atoi(value);
+	      break;
+	    case 51:
+	      unmuteSoundOnSwitch = atoi(value);
+	      break;
             } //switch
          }
          parameter++;
@@ -582,14 +643,12 @@ char* cSearchExt::BuildFile(const cEvent* pEvent) const
 
    if (useEpisode)
    {
-      char* pFile = NULL;
-      asprintf(&pFile, "%s~%s", pEvent->Title(), Subtitle);
-      if (file) free(file);
-      file = strdup(pFile);
-      free(pFile);
+     cString pFile = cString::sprintf("%s~%s", pEvent->Title(), Subtitle);
+     if (file) free(file);
+     file = strdup(pFile);
    }
-   else
-      asprintf(&file, "%s", pEvent->Title());
+   else if (pEvent->Title())
+     file = strdup(pEvent->Title());
 
    if (!isempty(directory))
    {
@@ -597,10 +656,10 @@ char* cSearchExt::BuildFile(const cEvent* pEvent) const
 
       cVarExpr varExprDir(directory);
       if (!varExprDir.DependsOnVar("%title%", pEvent) && !varExprDir.DependsOnVar("%subtitle%", pEvent))
-         asprintf(&pFile, "%s~%s", directory, file?file:"");
+         msprintf(&pFile, "%s~%s", directory, file?file:"");
       else
          // ignore existing title and subtitle in file if already used as variables in directory
-         asprintf(&pFile, "%s", directory);
+         msprintf(&pFile, "%s", directory);
 
       // parse the epxression and evaluate it
       cVarExpr varExprFile(pFile);
@@ -772,10 +831,11 @@ cEvent * cSearchExt::GetEventBySearchExt(const cSchedule *schedules, const cEven
    cEvent *pe = NULL;
    cEvent *p1 = NULL;
 
+   const cList<cEvent>* Events = schedules->Events();
    if (Start)
-      p1 = schedules->Events()->Next(Start);
+      p1 = Events->Next(Start);
    else
-      p1 = schedules->Events()->First();
+      p1 = Events->First();
 
    time_t tNow=time(NULL);
    char* szTest = NULL;
@@ -800,7 +860,7 @@ cEvent * cSearchExt::GetEventBySearchExt(const cSchedule *schedules, const cEven
    if (!useCase)
       ToLower(searchText);
 
-   for (cEvent *p = p1; p; p = schedules->Events()->Next(p))
+   for (cEvent *p = p1; p; p = Events->Next(p))
    {
       if(!p) 
       {
@@ -814,10 +874,10 @@ cEvent * cSearchExt::GetEventBySearchExt(const cSchedule *schedules, const cEven
       }
 
       // ignore events without title
-      if (!p->Title() || strlen(p->Title()) == 0)
+      if (!p->Title() || !*p->Title())
          continue;
         
-      asprintf(&szTest, "%s%s%s%s%s", (useTitle?(p->Title()?p->Title():""):""), (useSubtitle||useDescription)?"~":"",
+      msprintf(&szTest, "%s%s%s%s%s", (useTitle?(p->Title()?p->Title():""):""), (useSubtitle||useDescription)?"~":"",
                (useSubtitle?(p->ShortText()?p->ShortText():""):""),useDescription?"~":"",
                (useDescription?(p->Description()?p->Description():""):""));     
 	
@@ -891,7 +951,7 @@ cEvent * cSearchExt::GetEventBySearchExt(const cSchedule *schedules, const cEven
             }
          }
 
-         if (strlen(szTest) > 0)
+         if (*szTest)
          {
             if (!MatchesSearchMode(szTest, searchText, mode," ,;|~", fuzzyTolerance))
                continue;
@@ -1071,7 +1131,7 @@ void cSearchExt::CheckRepeatTimers(cSearchResults* pResults)
       cRecDone* firstRec = NULL;
       LogFile.Log(3,"get count recordings");
       records = RecsDone.GetCountRecordings(pEvent, this, &firstRec);
-      LogFile.Log(3,"rec  ordings: %d", records);	
+      LogFile.Log(3,"recordings: %d", records);	
       
       if (records > allowedRepeats) // already recorded
       {
@@ -1176,7 +1236,11 @@ void cSearchExt::CheckExistingRecordings(cSearchResults* pResults)
    int remain = pauseOnNrRecordings - num;
    for (pResultObj = pResults->First(); pResultObj; pResultObj = pResults->Next(pResultObj), remain--)
    {
-      if (!pResultObj->needsTimer) continue; // maybe already disabled because of done feature
+      if (!pResultObj->needsTimer) 
+	{
+	  remain++;
+	  continue; // maybe already disabled because of done feature
+	}
       pResultObj->needsTimer = (remain > 0); 		
       if (remain <= 0)	    
       {
@@ -1197,11 +1261,13 @@ bool cSearchExt::MatchesExtEPGInfo(const cEvent* e)
       int index = SearchExtCats.GetIndexFromID(SearchExtCat->id);
       if (index > -1)
          value = catvalues[index];
-      if (value && strlen(value) > 0)
+      if (value && SearchExtCat->searchmode >= 10 && atol(value) == 0) // numerical value != 0 ?
+	value = NULL;
+      if (value && *value)
       {
          char* testvalue = GetExtEPGValue(e, SearchExtCat);
          if (!testvalue)
-            return false;
+	   return (ignoreMissingEPGCats?true:false);
 
          // compare not case sensitive
          char* valueLower = strdup(value);
@@ -1296,6 +1362,17 @@ int cSearchExt::GetCountRecordings()
    return countRecs;
 }
 
+bool cSearchExt::IsActiveAt(time_t t)
+{
+  if (useAsSearchTimer == 0) return false;
+  if (useAsSearchTimer == 2)
+    {
+      if (useAsSearchTimerFrom > 0 && t < useAsSearchTimerFrom) return false;
+      if (useAsSearchTimerTil > 0 && t > useAsSearchTimerTil) return false;
+    } 
+  return true;
+}
+
 // -- cSearchExts ----------------------------------------------------------------
 bool cSearchExts::Load(const char *FileName)
 {
@@ -1458,3 +1535,26 @@ cSearchExts* cSearchExts::Clone()
    }
    return clonedList;
 }
+
+bool cSearchExts::CheckForAutoDelete(cSearchExt* SearchExt)
+{
+   if (!SearchExt || SearchExt->delMode == 0) return false;
+   
+   cRecDone* firstRec = NULL;
+   bool delSearch = false;
+   int recs = RecsDone.GetTotalCountRecordings(SearchExt, &firstRec);
+   if (SearchExt->delMode == 1 && SearchExt->delAfterCountRecs > 0)
+      delSearch = recs >= SearchExt->delAfterCountRecs;
+   if (SearchExt->delMode == 2 && SearchExt->delAfterDaysOfFirstRec && firstRec)
+      delSearch = (time(NULL) - firstRec->startTime) > SearchExt->delAfterDaysOfFirstRec * 24 * 60 * 60;
+   if (delSearch)
+   {
+      int DelID = SearchExt->ID;
+      LogFile.Log(1,"auto deleting search '%s' (ID: %d)", SearchExt->search, DelID);
+      cMutexLock SearchExtsLock(&SearchExts);
+      SearchExts.Del(SearchExt);
+      SearchExts.Save();
+      RecsDone.RemoveSearchID(DelID);
+   }
+   return delSearch;
+}
