@@ -1,5 +1,30 @@
+/*
+Copyright (C) 2004-2008 Christian Wieninger
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 2
+of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+Or, point your browser to http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+
+The author can be reached at cwieninger@gmx.de
+
+The project's page is at http://winni.vdr-developer.org/epgsearch
+*/
+
 #include <string>
 #include <set>
+#include <iomanip>
+#include <sstream>
 #include <vdr/plugin.h>
 #include <vdr/epg.h>
 #include "epgsearchtools.h"
@@ -12,10 +37,13 @@
 #include "epgsearchcats.h"
 #include "menu_dirselect.h"
 #include "menu_deftimercheckmethod.h"
+#include "conflictcheck.h"
 
 using std::string;
 using std::set;
 
+template< class Iter > Iter advance_copy( Iter it, std::size_t count = 1) { using std::advance; advance( it, count ); return it; }
+
 extern int updateForced;
 
 const char **cPluginEpgsearch::SVDRPHelpPages(void)
@@ -96,6 +124,9 @@ const char **cPluginEpgsearch::SVDRPHelpPages(void)
       "DEFT [ ID ]\n"
       "    Returns the ID of the default search template\n"
       "    or activates a search template with ID as default",
+      "LSCC [ REL ]\n"
+      "    Returns the current timer conflicts. With the option\n"
+      "    'REL' only relevant conflicts are listed",
       NULL
    };
    return HelpPages;
@@ -125,14 +156,7 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
          {
             cSearchExt *search = SearchExts.GetSearchFromID(atol(Option));
             if (search)
-            {
-               char* buffer = NULL;
-               asprintf(&buffer, "%s", search->ToText());
-               cString sBuffer = cString(buffer);
-               free(buffer);
-               return sBuffer;
-            }
-
+               return cString(search->ToText());
             else
             {
                ReplyCode = 901;
@@ -374,7 +398,6 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
                {
                   const cEvent* pEvent = result->event;
                   cTimer* Timer = new cTimer(pEvent);
-                  char *cmdbuf = NULL;
 
                   static char bufStart[25];
                   static char bufEnd[25];
@@ -395,10 +418,14 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
                   strftime(bufStart, sizeof(bufStart), "%H%M", localtime_r(&start, &tm_r));
                   strftime(bufEnd, sizeof(bufEnd), "%H%M", localtime_r(&stop, &tm_r));
 
-                  asprintf(&cmdbuf, "NEWT %d:%d:%s:%s:%s:%d:%d:%s:%s", 
+		  cString cmdbuf = cString::sprintf("NEWT %d:%d:%s:%s:%s:%d:%d:%s:%s", 
                            Flags,
                            Timer->Channel()->Number(),
+#if VDRVERSNUM < 10503
                            *Timer->PrintDay(start, Timer->WeekDays()),
+#else
+                           *Timer->PrintDay(start, Timer->WeekDays(), true),
+#endif
                            bufStart,
                            bufEnd,
                            search->Priority, 
@@ -408,7 +435,6 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
 				 
 		      
                   sBuffer += string(cmdbuf) + string(results->Next(result)?"\n":"");
-                  free(cmdbuf);
                   delete(Timer);
                   result = results->Next(result);
                }
@@ -496,7 +522,6 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
          {
             const cEvent* pEvent = result->event;
             cTimer* Timer = new cTimer(pEvent);
-            char *cmdbuf = NULL;
 		
             static char bufStart[25];
             static char bufEnd[25];
@@ -530,7 +555,7 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
             string shorttext = pEvent->ShortText()?ReplaceAll(pEvent->ShortText(), "|", "!^pipe!^"):"";
             shorttext = ReplaceAll(shorttext, ":", "|");
 
-            asprintf(&cmdbuf, "%d:%u:%s:%s:%ld:%ld:%s:%ld:%ld:%s:%d", 
+            cString cmdbuf = cString::sprintf("%d:%u:%s:%s:%ld:%ld:%s:%ld:%ld:%s:%d", 
                      result->search->ID,
                      pEvent->EventID(),
                      title.c_str(),
@@ -544,7 +569,6 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
                      timerMode);				 
 		
             sBuffer += string(cmdbuf) + string(pCompleteSearchResults->Next(result)?"\n":"");
-            free(cmdbuf);
             delete(Timer);
             result = pCompleteSearchResults->Next(result);
          }
@@ -593,14 +617,7 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
       {
          cChannelGroup *changrp = ChannelGroups.GetGroupByName(Option);
          if (changrp)
-         {
-            char* buffer = NULL;
-            asprintf(&buffer, "%s", changrp->ToText());
-            cString sBuffer = cString(buffer);
-            free(buffer);
-            return sBuffer;
-         }
-	    
+            return cString(changrp->ToText());	    
          else
          {
             ReplyCode = 901;
@@ -781,14 +798,7 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
          {
             cBlacklist *blacklist = Blacklists.GetBlacklistFromID(atol(Option));
             if (blacklist)
-            {
-               char* buffer = NULL;
-               asprintf(&buffer, "%s", blacklist->ToText());
-               cString sBuffer = cString(buffer);
-               free(buffer);
-               return sBuffer;
-            }
-
+               return cString(blacklist->ToText());
             else
             {
                ReplyCode = 901;
@@ -929,14 +939,7 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
             int index = SearchExtCats.GetIndexFromID(atoi(Option));
             if (index >= 0) SearchExtCat = SearchExtCats.Get(index);
             if (SearchExtCat)
-            {
-               char* buffer = NULL;
-               asprintf(&buffer, "%s", SearchExtCat->ToText());
-               cString sBuffer = cString(buffer);
-               free(buffer);
-               return sBuffer;
-            }
-
+               return cString(SearchExtCat->ToText());
             else
             {
                ReplyCode = 901;
@@ -1058,14 +1061,7 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
          {
             cSearchExt *search = SearchTemplates.GetSearchFromID(atol(Option));
             if (search)
-            {
-               char* buffer = NULL;
-               asprintf(&buffer, "%s", search->ToText());
-               cString sBuffer = cString(buffer);
-               free(buffer);
-               return sBuffer;
-            }
-
+               return cString(search->ToText());
             else
             {
                ReplyCode = 901;
@@ -1214,5 +1210,58 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
          return cString::sprintf("%d", EPGSearchConfig.DefSearchTemplateID);
    }
 
+   // ---------------------------------
+   // timer conflicts
+   else if (strcasecmp(Command, "LSCC") == 0) 
+   {
+     bool relOnly = false;
+     if (*Option && strcasecmp(Option, "REL") == 0)
+       relOnly = true;
+
+     cConflictCheck conflictCheck;
+     conflictCheck.Check(); 
+     
+     if ((relOnly && conflictCheck.numConflicts > 0) ||
+	 conflictCheck.relevantConflicts > 0)
+       {
+         string sBuffer;
+	 cList<cConflictCheckTime>* failedList = conflictCheck.GetFailed();
+	 for(cConflictCheckTime* ct = failedList->First(); ct; ct = failedList->Next(ct))
+	   {
+	     if (relOnly && ct->ignore) continue;
+
+	     std::ostringstream conflline;
+	     conflline << ct->evaltime << ":";
+	     std::set<cConflictCheckTimerObj*,TimerObjSort>::iterator it;
+
+	     std::ostringstream timerparts;
+	     for (it = ct->failedTimers.begin(); it != ct->failedTimers.end(); it++) 
+	       {
+		 if (relOnly && (*it)->ignore) continue;
+		 std::ostringstream timerpart;
+		 int recPart = (*it)->recDuration * 100 / ((*it)->stop - (*it)->start);
+		 timerpart << (*it)->timer->Index()+1 << "|" << recPart << "|";
+		 std::set<cConflictCheckTimerObj*,TimerObjSort>::iterator itcc;
+		 if ((*it)->concurrentTimers)
+		   {
+		   std::ostringstream cctimers;
+		   for (itcc = (*it)->concurrentTimers->begin(); itcc != (*it)->concurrentTimers->end(); itcc++) 
+		     cctimers << (cctimers.str().empty()?"":"#") << (*itcc)->timer->Index()+1;
+		   timerpart << cctimers.str();
+		   }
+		 timerparts << (timerparts.str().empty()?"":":") << timerpart.str();
+	       }
+	     conflline << timerparts.str();
+	     sBuffer += conflline.str() + "\n";
+	   }
+	 return sBuffer.c_str();
+       }
+     else
+       {
+         ReplyCode = 901;
+         return cString("no conflicts found");	    	
+       }
+   }
+
    return NULL;
 }
