@@ -1,10 +1,25 @@
 /*
- * menu_searchresults.c: A plugin for the Video Disk Recorder
- *
- * See the README file for copyright information and how to reach the author.
- *
- * $Id$
- */
+Copyright (C) 2004-2008 Christian Wieninger
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 2
+of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+Or, point your browser to http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+
+The author can be reached at cwieninger@gmx.de
+
+The project's page is at http://winni.vdr-developer.org/epgsearch
+*/
 
 #include <vector>
 #include "menu_searchresults.h"
@@ -22,9 +37,12 @@
 #include "menu_conflictcheck.h"
 #include "uservars.h"
 #include "menu_deftimercheckmethod.h"
+#include "afuzzy.h"
+#include "timerstatus.h"
 
 const char* ButtonBlue[3] = {NULL, NULL, NULL};
 extern int gl_InfoConflict;
+extern bool isUTF8;
 
 static int CompareRecording(const void *p1, const void *p2)
 {
@@ -61,13 +79,25 @@ bool cMenuSearchResultsItem::Update(bool Force)
     
    if (Force || timerMatch != OldTimerMatch) 
    {
-      char t[2]="",v[2]="",r[2]="";
-      char szStatus[4] = "";
+     char t[Utf8BufSize(2)]="",v[Utf8BufSize(2)]="",r[Utf8BufSize(2)]="";
+     char szStatus[Utf8BufSize(4)] = "";
       if (EPGSearchConfig.WarEagle)
       {
-         sprintf(t, "%c", event && hasMatch ? (timerMatch == tmFull) ? ((timer && timer->Recording())?249:253) : 't' : ' ');
-         sprintf(v, "%c", event && event->Vps() && (event->Vps() - event->StartTime()) ? 'V' : ' ');
-         sprintf(r, "%c", event && event->IsRunning() ? 251 : ' ');
+	if (!isUTF8)
+	  {
+	    sprintf(t, "%c", event && hasMatch ? (timerMatch == tmFull) ? ((timer && timer->Recording())?ICON_REC:ICON_CLOCK) : ICON_CLOCK_HALF : ' ');
+	    sprintf(v, "%c", event && event->Vps() && (event->Vps() - event->StartTime()) ? ICON_VPS : ' ');
+	    sprintf(r, "%c", event && event->IsRunning() ? ICON_RUNNING : ' ');
+	  }
+	else
+	  {
+#if defined(__GNUC__) && __GNUC__ < 3 && __GNUC_MINOR__ < 96 
+#else
+	    sprintf(t, "%s", (event && hasMatch ? (timerMatch == tmFull) ? ((timer && timer->Recording())?ICON_REC_UTF8:ICON_CLOCK_UTF8) : ICON_CLOCK_HALF_UTF8 : " "));
+	    sprintf(v, "%s", event && event->Vps() && (event->Vps() - event->StartTime()) ? ICON_VPS_UTF8 : " ");
+	    sprintf(r, "%s", (event && event->IsRunning() ? ICON_RUNNING_UTF8 : " "));
+#endif
+	  }
       }
       else
       {
@@ -85,7 +115,8 @@ bool cMenuSearchResultsItem::Update(bool Force)
       free(buffer);
       buffer = tmp;
 
-      if (!strcasestr(buffer, "%subtitle%")) // make sure, there is a subtitle
+      if (!strcasestr(buffer, "%subtitle%") && cTemplFile::GetTemplateByName("MenuFavorites") != menuTemplate) 
+	// make sure, there is a subtitle
          buffer = strreplacei(buffer, "%title%", "%title% ~ %subtitle%");
       if (episodeOnly)
          buffer = strreplacei(buffer, "%title%", "");
@@ -96,10 +127,10 @@ bool cMenuSearchResultsItem::Update(bool Force)
       free(buffer);
       buffer = tmp;
 
-      buffer = strreplacei(buffer, "%status%", szStatus);
-      buffer = strreplacei(buffer, "%t_status%", t);
-      buffer = strreplacei(buffer, "%v_status%", v);
-      buffer = strreplacei(buffer, "%r_status%", r);
+      buffer = strreplacei(buffer, "$status$", szStatus);
+      buffer = strreplacei(buffer, "$t_status$", t);
+      buffer = strreplacei(buffer, "$v_status$", v);
+      buffer = strreplacei(buffer, "$r_status$", r);
 
       buffer = FixSeparators(buffer, '~');
       buffer = FixSeparators(buffer, ':');
@@ -135,9 +166,10 @@ const cEvent *cMenuSearchResults::scheduleEventInfo = NULL;
 cMenuSearchResults::cMenuSearchResults(cMenuTemplate* MenuTemplate)
    :cOsdMenu("", MenuTemplate->Tab(0), MenuTemplate->Tab(1), MenuTemplate->Tab(2), MenuTemplate->Tab(3), MenuTemplate->Tab(4))
 {
-   menuTemplate = MenuTemplate;
-   modeYellow = showTitleEpisode;
-   ignoreRunning = false;
+  helpKeys = -1;
+  menuTemplate = MenuTemplate;
+  modeYellow = showTitleEpisode;
+  ignoreRunning = false;
 }
 
 int cMenuSearchResults::GetTab(int Tab)
@@ -168,10 +200,12 @@ eOSState cMenuSearchResults::Record(void)
          int tm = tmNone;
          cTimer *timer = Timers.GetMatch(item->event, &tm);
          if (timer)
-            if (EPGSearchConfig.useVDRTimerEditMenu)
+	   {
+	     if (EPGSearchConfig.useVDRTimerEditMenu)
                return AddSubMenu(new cMenuEditTimer(timer));
-            else
+	     else
                return AddSubMenu(new cMenuMyEditTimer(timer, false, item->event));
+	   }
       }
 
       cTimer *timer = new cTimer(item->event);
@@ -192,12 +226,12 @@ eOSState cMenuSearchResults::Record(void)
       else
       {
          string fullaux = "";
+         string aux = "";
          if (item->event)
          {
             const cEvent* event = item->event;
             int bstart = event->StartTime() - timer->StartTime();
             int bstop = timer->StopTime() - event->EndTime();
-            string aux = "";
             int checkmode = DefTimerCheckModes.GetMode(timer->Channel());
             aux = UpdateAuxValue(aux, "channel", NumToString(timer->Channel()->Number()) + " - " + CHANNELNAME(timer->Channel()));
             aux = UpdateAuxValue(aux, "update", checkmode);
@@ -206,8 +240,16 @@ eOSState cMenuSearchResults::Record(void)
             aux = UpdateAuxValue(aux, "bstop", bstop);
             fullaux = UpdateAuxValue(fullaux, "epgsearch", aux);
          }
+
+#ifdef USE_PINPLUGIN
+         aux = "";
+         aux = UpdateAuxValue(aux, "protected", timer->FskProtection() ? "yes" : "no");
+         fullaux = UpdateAuxValue(fullaux, "pin-plugin", aux);
+#endif
+
          SetAux(timer, fullaux);
          Timers.Add(timer);
+	 gl_timerStatusMonitor->SetConflictCheckAdvised(); 
          timer->Matches();
          Timers.SetModified();
          LogFile.iSysLog("timer %s added (active)", *timer->ToDescr());
@@ -232,7 +274,7 @@ eOSState cMenuSearchResults::Switch(void)
       if (channel && cDevice::PrimaryDevice()->SwitchChannel(channel, true))
          return osEnd;
    }
-   Skins.Message(mtInfo, tr("Can't switch channel!")); 
+   Skins.Message(mtInfo, trVDR("Can't switch channel!")); 
    return osContinue;
 }
 
@@ -342,7 +384,7 @@ eOSState cMenuSearchResults::ProcessKey(eKeys Key)
             if(!HasSubMenu()) 
             {
                toggleKeys = 1 - toggleKeys;
-               SetHelpKeys();
+               SetHelpKeys(true);
             }
             state = osContinue;
             break;
@@ -353,6 +395,7 @@ eOSState cMenuSearchResults::ProcessKey(eKeys Key)
             state = OnYellow();
             break;
          case kOk:
+      case kInfo:
             if(HasSubMenu()) 
             {
                state = cOsdMenu::ProcessKey(Key);
@@ -438,11 +481,10 @@ bool cMenuSearchResultsForSearch::BuildList()
    }
    if (Count())
       SetCurrent(Get(0));
-   SetHelpKeys();  
-   char* szTitle = NULL;
-   asprintf(&szTitle, "%d %s - %s", Count(), tr("Search results"), searchExt->search);
+   SetHelpKeys(true);  
+
+   cString szTitle = cString::sprintf("%d %s - %s", Count(), tr("Search results"), searchExt->search);
    SetTitle(szTitle);
-   free(szTitle);
 
    SetCurrent(Get(current));
    Display();
@@ -450,20 +492,31 @@ bool cMenuSearchResultsForSearch::BuildList()
    return hasResults;
 }
 
-void cMenuSearchResultsForSearch::SetHelpKeys()
+void cMenuSearchResultsForSearch::SetHelpKeys(bool Force)
 {
    cMenuSearchResultsItem *item = (cMenuSearchResultsItem *)Get(Current());
-   bool hasTimer = item && item->Selectable() && item->timerMatch == tmFull;
+  int NewHelpKeys = 0;
+  if (item) {
+    if (item->Selectable() && item->timerMatch == tmFull)	
+      NewHelpKeys = 2;
+    else
+      NewHelpKeys = 1;
+  }
 
-   ModeBlueSR nextModeBlue = (ModeBlueSR)(((int)modeBlue+1)%3);
-   if (nextModeBlue == showTimerPreview && 
-       (searchExt->useAsSearchTimer == 0 || searchExt->avoidRepeats == 0))
-      nextModeBlue = (ModeBlueSR)(((int)nextModeBlue+1)%3);
-
-   if (toggleKeys==0)
-      SetHelp((EPGSearchConfig.redkeymode==0?(hasTimer?tr("Button$Timer"):tr("Button$Record")):tr("Button$Commands")), m_bSort? tr("Button$by channel"):tr("Button$by time"), modeYellow==showTitleEpisode?tr("Button$Episode"):tr("Button$Title"), ButtonBlue[(int)nextModeBlue]);
-   else
-      SetHelp((EPGSearchConfig.redkeymode==1?(hasTimer?tr("Button$Timer"):tr("Button$Record")):tr("Button$Commands")), m_bSort? tr("Button$by channel"):tr("Button$by time"), modeYellow==showTitleEpisode?tr("Button$Episode"):tr("Button$Title"), ButtonBlue[(int)nextModeBlue]);
+  bool hasTimer = (NewHelpKeys == 2);
+  if (NewHelpKeys != helpKeys || Force)
+    { 
+      ModeBlueSR nextModeBlue = (ModeBlueSR)(((int)modeBlue+1)%3);
+      if (nextModeBlue == showTimerPreview && 
+	  (searchExt->useAsSearchTimer == 0 || searchExt->avoidRepeats == 0))
+	nextModeBlue = (ModeBlueSR)(((int)nextModeBlue+1)%3);
+      
+      if (toggleKeys==0)
+	SetHelp((EPGSearchConfig.redkeymode==0?(hasTimer?trVDR("Button$Timer"):trVDR("Button$Record")):tr("Button$Commands")), m_bSort? tr("Button$by channel"):tr("Button$by time"), modeYellow==showTitleEpisode?tr("Button$Episode"):tr("Button$Title"), ButtonBlue[(int)nextModeBlue]);
+      else
+	SetHelp((EPGSearchConfig.redkeymode==1?(hasTimer?trVDR("Button$Timer"):trVDR("Button$Record")):tr("Button$Commands")), m_bSort? tr("Button$by channel"):tr("Button$by time"), modeYellow==showTitleEpisode?tr("Button$Episode"):tr("Button$Title"), ButtonBlue[(int)nextModeBlue]);
+      helpKeys = NewHelpKeys;
+    }
 }
 
 eOSState cMenuSearchResultsForSearch::ProcessKey(eKeys Key)
@@ -544,10 +597,8 @@ bool cMenuSearchResultsForBlacklist::BuildList()
    if (Count())
       SetCurrent(Get(0));
    SetHelpKeys();  
-   char* szTitle = NULL;
-   asprintf(&szTitle, "%d %s - %s", Count(), tr("Blacklist results"), blacklist->search);
+   cString szTitle = cString::sprintf("%d %s - %s", Count(), tr("Blacklist results"), blacklist->search);
    SetTitle(szTitle);
-   free(szTitle);
 
    SetCurrent(Get(current));
    Display();
@@ -581,20 +632,31 @@ eOSState cMenuSearchResultsForBlacklist::ProcessKey(eKeys Key)
    return state;
 }
 
-void cMenuSearchResultsForBlacklist::SetHelpKeys()
+void cMenuSearchResultsForBlacklist::SetHelpKeys(bool Force)
 {
-   cMenuSearchResultsItem *item = (cMenuSearchResultsItem *)Get(Current());
-   bool hasTimer = item && item->Selectable() && item->timerMatch == tmFull;
+  cMenuSearchResultsItem *item = (cMenuSearchResultsItem *)Get(Current());
+  int NewHelpKeys = 0;
+  if (item) {
+    if (item->Selectable() && item->timerMatch == tmFull)	
+      NewHelpKeys = 2;
+    else
+      NewHelpKeys = 1;
+  }
 
-   ModeBlueSR nextModeBlue = (ModeBlueSR)(((int)modeBlue+1)%3);
-   if (nextModeBlue == showTimerPreview)
-      nextModeBlue = (ModeBlueSR)(((int)nextModeBlue+1)%3);
-
-   if (toggleKeys==0)
-      SetHelp((EPGSearchConfig.redkeymode==0?(hasTimer?tr("Button$Timer"):tr("Button$Record")):tr("Button$Commands")), m_bSort? tr("Button$by channel"):tr("Button$by time"), modeYellow==showTitleEpisode?tr("Button$Episode"):tr("Button$Title"), NULL);
-   else
-      SetHelp((EPGSearchConfig.redkeymode==1?(hasTimer?tr("Button$Timer"):tr("Button$Record")):tr("Button$Commands")), m_bSort? tr("Button$by channel"):tr("Button$by time"), modeYellow==showTitleEpisode?tr("Button$Episode"):tr("Button$Title"), NULL);
+  bool hasTimer = (NewHelpKeys == 2);
+  if (NewHelpKeys != helpKeys || Force)
+    { 
 
+      ModeBlueSR nextModeBlue = (ModeBlueSR)(((int)modeBlue+1)%3);
+      if (nextModeBlue == showTimerPreview)
+	nextModeBlue = (ModeBlueSR)(((int)nextModeBlue+1)%3);
+      
+      if (toggleKeys==0)
+	SetHelp((EPGSearchConfig.redkeymode==0?(hasTimer?trVDR("Button$Timer"):trVDR("Button$Record")):tr("Button$Commands")), m_bSort? tr("Button$by channel"):tr("Button$by time"), modeYellow==showTitleEpisode?tr("Button$Episode"):tr("Button$Title"), NULL);
+      else
+	SetHelp((EPGSearchConfig.redkeymode==1?(hasTimer?trVDR("Button$Timer"):trVDR("Button$Record")):tr("Button$Commands")), m_bSort? tr("Button$by channel"):tr("Button$by time"), modeYellow==showTitleEpisode?tr("Button$Episode"):tr("Button$Title"), NULL);
+      helpKeys = NewHelpKeys;
+    }
 }
 
 // --- cMenuSearchResultsForQuery -------------------------------------------------------
@@ -666,11 +728,43 @@ bool cMenuSearchResultsForRecs::BuildList()
    int current = Current();
    Clear();
    for (cRecording *recording = Recordings.First(); recording; recording = Recordings.Next(recording)) {
-      if (strstr(recording->Title(), searchExt->search) ||
-          strstr(searchExt->search, recording->Title())) {
-         pArray = (cRecording **)realloc(pArray, (num + 1) * sizeof(cRecording *));
-         pArray[num++] = recording;
-      }
+     const cRecordingInfo *recInfo = recording->Info();
+     if (!recInfo) continue;
+     string s1 = (recInfo && recInfo->Title())?recInfo->Title():"";
+     string s2 = searchExt->search;
+     if (s1.empty() || s2.empty()) continue;
+
+     // tolerance for fuzzy searching: 90% of the shorter text lenght, but atleast 1
+     int tolerance = std::max(1, (int)std::min(s1.size(), s2.size()) / 10); 
+
+     bool match = FindIgnoreCase(s1, s2) >= 0 ||
+       FindIgnoreCase(s2, s1) >= 0;
+
+     if (!match)
+       {
+	 AFUZZY af = { NULL, NULL, NULL, NULL, NULL, NULL };
+	 if (s1.size() > 32) s1 = s1.substr(0, 32);
+	 afuzzy_init(s1.c_str(), tolerance, 0, &af);
+	 /* Checking substring */
+	 int res = afuzzy_checkSUB(s2.c_str(), &af);
+	 afuzzy_free(&af);
+	 match = (res > 0);
+       }
+     if (!match)
+       {
+	 AFUZZY af = { NULL, NULL, NULL, NULL, NULL, NULL };
+	 if (s2.size() > 32) s2 = s2.substr(0, 32);
+	 afuzzy_init(s2.c_str(), tolerance, 0, &af);
+	 /* Checking substring */
+	 int res = afuzzy_checkSUB(s1.c_str(), &af);
+	 afuzzy_free(&af);
+	 match = (res > 0);
+       }
+
+     if (match) {
+       pArray = (cRecording **)realloc(pArray, (num + 1) * sizeof(cRecording *));
+       pArray[num++] = recording;
+     }
    }
 
    qsort(pArray, num, sizeof(cRecording *), CompareRecording);
@@ -714,7 +808,12 @@ eOSState cMenuSearchResultsForRecs::ProcessKey(eKeys Key)
    if (state == osUnknown) 
    {
       if (Key == kOk)
-         state = Play();
+	{
+	  if (Count() > 0)
+	    state = Play();
+	  else
+	    state = osBack;
+	}
       else
          state = osContinue;
    }
@@ -732,21 +831,30 @@ cMenuSearchResultsForList::cMenuSearchResultsForList(cSearchResults& SearchResul
 
    BuildList();
     
-   char* szTitle = NULL;
-   asprintf(&szTitle, Title, Count());
+   cString szTitle = cString::sprintf(Title, Count());
    SetTitle(szTitle);
-   free(szTitle);
 }
 
-void cMenuSearchResultsForList::SetHelpKeys()
+void cMenuSearchResultsForList::SetHelpKeys(bool Force)
 {
-   cMenuSearchResultsItem *item = (cMenuSearchResultsItem *)Get(Current());
-   bool hasTimer = item && item->Selectable() && item->timerMatch == tmFull;
+  cMenuSearchResultsItem *item = (cMenuSearchResultsItem *)Get(Current());
+  int NewHelpKeys = 0;
+  if (item) {
+    if (item->Selectable() && item->timerMatch == tmFull)	
+      NewHelpKeys = 2;
+    else
+      NewHelpKeys = 1;
+  }
 
-   if (toggleKeys==0)
-      SetHelp((EPGSearchConfig.redkeymode==0?(hasTimer?tr("Button$Timer"):tr("Button$Record")):tr("Button$Commands")), m_bSort? tr("Button$by channel"):tr("Button$by time"), modeYellow==showTitleEpisode?tr("Button$Episode"):tr("Button$Title"), ButtonBlue[0]);
-   else
-      SetHelp((EPGSearchConfig.redkeymode==1?(hasTimer?tr("Button$Timer"):tr("Button$Record")):tr("Button$Commands")), m_bSort? tr("Button$by channel"):tr("Button$by time"), modeYellow==showTitleEpisode?tr("Button$Episode"):tr("Button$Title"), ButtonBlue[0]);
+  bool hasTimer = (NewHelpKeys == 2);
+  if (NewHelpKeys != helpKeys || Force)
+    { 
+      if (toggleKeys==0)
+	SetHelp((EPGSearchConfig.redkeymode==0?(hasTimer?trVDR("Button$Timer"):trVDR("Button$Record")):tr("Button$Commands")), m_bSort? tr("Button$by channel"):tr("Button$by time"), modeYellow==showTitleEpisode?tr("Button$Episode"):tr("Button$Title"), ButtonBlue[0]);
+      else
+	SetHelp((EPGSearchConfig.redkeymode==1?(hasTimer?trVDR("Button$Timer"):trVDR("Button$Record")):tr("Button$Commands")), m_bSort? tr("Button$by channel"):tr("Button$by time"), modeYellow==showTitleEpisode?tr("Button$Episode"):tr("Button$Title"), ButtonBlue[0]);
+      helpKeys = NewHelpKeys;
+    }
 }
 
 bool cMenuSearchResultsForList::BuildList()
