@@ -1,12 +1,29 @@
 /*
- * menu_whatson.c: A plugin for the Video Disk Recorder
- *
- * See the README file for copyright information and how to reach the author.
- *
- * $Id$
- */
+Copyright (C) 2004-2008 Christian Wieninger
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 2
+of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+Or, point your browser to http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+
+The author can be reached at cwieninger@gmx.de
+
+The project's page is at http://winni.vdr-developer.org/epgsearch
+*/
 
 #include <vector>
+#include <iostream>
+#include <sstream>
 #include <string>
 #include "uservars.h"
 #include "menu_whatson.h"
@@ -26,6 +43,7 @@
 #include "menu_conflictcheck.h"
 #include "templatefile.h"
 #include "menu_deftimercheckmethod.h"
+#include "timerstatus.h"
 
 #define HOURS(x) ((x)/100)
 #define MINUTES(x) ((x)%100)
@@ -34,6 +52,7 @@
 #define HOURS2SECS(x) (x*60*60)
 
 extern int exitToMainMenu;
+extern bool isUTF8;
 int gl_InfoConflict = 0;
 
 // --- cMenuMyScheduleItem ------------------------------------------------------
@@ -69,80 +88,146 @@ bool cMenuMyScheduleItem::Update(bool Force)
     
    if (Force || timerMatch != OldTimerMatch || inSwitchList != OldInSwitchList) 
    {
-      char szProgressPart[12] = "";
+     char szProgressPart[Utf8BufSize(12)] = "";
       char szProgressPartT2S[12] = "";
+      time_t now = time(NULL);
       if (channel)
       {
          if (event)
          {
-            int frac = 0;
-            if (mode == showNow)
-               frac = (int)roundf( (float)(time(NULL) - event->StartTime()) / (float)(event->Duration()) * 8.0 );
-            if (mode == showNext)
-               frac = (int)roundf( (float)(1 - min(float(30*60), float(event->StartTime() - time(NULL))) / (30*60)) * 8.0 );
-            frac = min(8,max(0, frac));
-		
-            char szProgressT2S[9] = "";
-            for(int i = 0; i < frac; i++) szProgressT2S[i] = '|';
-            szProgressT2S[frac]=0;
-            sprintf(szProgressPartT2S, "%c%-8s%c", '[', szProgressT2S, ']');
-
-            char szProgress[9] = {130,131,131,131,131,131,131,132,0};
-            for(int i = 0; i < frac; i++) szProgress[i] = 127;
-            sprintf(szProgressPart, "%-8s", szProgress);
+	   time_t startTime = event->StartTime();
+	   if ((now - event->StartTime()) >= 0 || strstr(menutemplate, "%time%") != NULL)
+            {
+	      int frac = 0;
+	      if (mode == showNow)
+		{
+		  int dur = event->Duration();
+		  frac = ((now - startTime) * 8 + (dur >> 1)) / dur;
+		}
+	      if (mode == showNext)
+		frac = (  ( 30*60 - min((long int)30*60, startTime - now) ) * 8 + 15*60  ) / (30*60);
+	      
+	      frac = min(8,max(0, frac));
+	      
+	      szProgressPartT2S[0] = '[';
+	      memset(szProgressPartT2S + 1,'|',frac);
+	      memset(szProgressPartT2S + 1 + frac ,' ', 8 - frac);
+	      szProgressPartT2S[9] = ']';
+	      szProgressPartT2S[10] = 0;
+	      
+	      if (!isUTF8)
+		{
+		  szProgressPart[0] = ICON_BAR_OPEN;
+		  memset(szProgressPart + 1, ICON_BAR_EMPTY, 6);
+		  szProgressPart[7] = ICON_BAR_CLOSE;
+		  szProgressPart[8] = 0;
+		  memset(szProgressPart, ICON_BAR_FULL, frac);
+		}
+	      else
+		{
+#if defined(__GNUC__) && __GNUC__ < 3 && __GNUC_MINOR__ < 96 
+#else
+		  std::stringstream buffer;
+		  buffer << ICON_BAR_OPEN_UTF8;
+		  for(int i=0;i<8;i++) buffer << (i<frac?ICON_BAR_FULL_UTF8:ICON_BAR_EMPTY_UTF8);
+		  buffer << ICON_BAR_CLOSE_UTF8;
+		  char* temp = strdup(buffer.str().c_str());
+		  sprintf(szProgressPart, "%s", temp);
+		  free(temp);
+#endif
+		}
+            }
+            else
+            {
+	      strncpy(szProgressPart, *event->GetTimeString(), 12);
+	      szProgressPart[11] = 0;
+	      memcpy(szProgressPartT2S, szProgressPart, 12);
+            }
          }
       }
       
-      char t[2]="",v[2]="",r[2]="";
-      char szStatus[4] = "";
+      char t[Utf8BufSize(2)],v[Utf8BufSize(2)],r[Utf8BufSize(2)];
+      char szStatus[Utf8BufSize(4)];
+      szStatus[3] = 0;
+      t[1]=v[1]=r[1] = 0; 
+
       if (EPGSearchConfig.WarEagle)
       {
-         sprintf(t, "%c", event && hasMatch ? (timerMatch == tmFull) ? ((timer && timer->Recording())?249:253) : 't' : ' ');
-         sprintf(v, "%c", event && event->Vps() && (event->Vps() - event->StartTime()) ? 'V' : ' ');
-         sprintf(r, "%c", event && event->IsRunning() ? 251 : ' ');
+	if (!isUTF8)
+	  {
+	    t[0] = event && hasMatch ? (timerMatch == tmFull) ? ((timer && timer->Recording())?ICON_REC:ICON_CLOCK) : ICON_CLOCK_HALF : ' ';
+	    v[0] = event && event->Vps() && (event->Vps() - event->StartTime()) ? ICON_VPS : ' ';
+	    r[0] = event && event->IsRunning() ? ICON_RUNNING : ' ';
+	  }
+	else
+	  {
+#if defined(__GNUC__) && __GNUC__ < 3 && __GNUC_MINOR__ < 96 
+#else
+	    sprintf(t, "%s", (event && hasMatch ? (timerMatch == tmFull) ? ((timer && timer->Recording())?ICON_REC_UTF8:ICON_CLOCK_UTF8) : ICON_CLOCK_HALF_UTF8 : " "));
+	    sprintf(v, "%s", event && event->Vps() && (event->Vps() - event->StartTime()) ? ICON_VPS_UTF8 : " ");
+	    sprintf(r, "%s", (event && event->IsRunning() ? ICON_RUNNING_UTF8 : " "));
+#endif
+	  }
       }
       else
       {
-         sprintf(t, "%c", event && hasMatch ? (timerMatch == tmFull) ? ((timer && timer->Recording())?'R':'T') : 't' : ' ');
-         sprintf(v, "%c", event && event->Vps() && (event->Vps() - event->StartTime()) ? 'V' : ' ');
-         sprintf(r, "%c", event && event->IsRunning() ? '*' : ' ');
+	t[0] = event && hasMatch ? (timerMatch == tmFull) ? ((timer && timer->Recording())?'R':'T') : 't' : ' ';
+	v[0] = event && event->Vps() && (event->Vps() - event->StartTime()) ? 'V' : ' ';
+	r[0] = event && event->IsRunning() ? '*' : ' ';
       }
 
       if (event && inSwitchList)
       {
          cSwitchTimer* s = SwitchTimers.InSwitchList(event);
-         sprintf(t, "%c", (s && s->announceOnly)?'s':'S');
+	 t[0] = (s && s->announceOnly)?'s':'S';
       }
-      sprintf(szStatus, "%s%s%s", t,v,r);
+      if (EPGSearchConfig.WarEagle && isUTF8)
+	{
+	  std::stringstream buffer;
+	  buffer << t << v << r;
+	  char* temp = strdup(buffer.str().c_str());
+	  sprintf(szStatus, "%s", temp);
+	  free(temp);
+	}
+      else
+	{
+	  szStatus[0] = t[0];
+	  szStatus[1] = v[0];
+	  szStatus[2] = r[0];
+	}
 
       char* buffer = strdup(menutemplate);
-      char* tmp = strreplaceall(buffer, '|', "\t");
-      free(buffer);
-      buffer = tmp;
+      strreplace(buffer, '|', '\t');
+
+      char* title = NULL; 
+      msprintf(&title, "%s", event?event->Title():tr(">>> no info! <<<"));
+      title = strreplacei(title, ":", "%colon%"); // asume a title has the form "a?b:c", 
+      // we need to replace the colon to avoid misinterpretation the expression as a condition
+      buffer = strreplacei(buffer, "%title%", title);
+      free(title);
 
-      buffer = strreplacei(buffer, "%title%", event?event->Title():tr(">>> no info! <<<"));
       if (channel)
       {
          char szChannelNr[6] = "";
-         if (channel)
-            sprintf(szChannelNr, "%d", channel->Number());
+	 sprintf(szChannelNr, "%d", channel->Number());
          buffer = strreplacei(buffer, "%chnr%", szChannelNr);
-         buffer = strreplacei(buffer, "%chsh%", channel?channel->ShortName(true):"");
-         buffer = strreplacei(buffer, "%chlng%", channel?channel->Name():"");
+         buffer = strreplacei(buffer, "%chsh%", channel->ShortName(true));
+         buffer = strreplacei(buffer, "%chlng%", channel->Name());
          buffer = strreplacei(buffer, "%progr%", szProgressPart);
          buffer = strreplacei(buffer, "%progrT2S%", szProgressPartT2S);
       }
 
       // parse the epxression and evaluate it
       cVarExpr varExpr(buffer);
-      tmp = strdup(varExpr.Evaluate(event).c_str());
+      char* tmp = strdup(varExpr.Evaluate(event).c_str());
       free(buffer);
       buffer = tmp;
+      
+      buffer = strreplacei(buffer, "$status$", szStatus);
+      buffer = strreplacei(buffer, "$t_status$", t);
+      buffer = strreplacei(buffer, "$v_status$", v);
+      buffer = strreplacei(buffer, "$r_status$", r);
 
-      buffer = strreplacei(buffer, "%status%", szStatus);
-      buffer = strreplacei(buffer, "%t_status%", t);
-      buffer = strreplacei(buffer, "%v_status%", v);
-      buffer = strreplacei(buffer, "%r_status%", r);
 
 
       buffer = FixSeparators(buffer, '~');
@@ -170,36 +255,49 @@ showMode cMenuWhatsOnSearch::currentShowMode = showNow;
 cChannel *cMenuWhatsOnSearch::scheduleChannel = NULL;
 extern const char *ShowModes[];
 cList<cShowMode> cMenuWhatsOnSearch::showModes;
+time_t cMenuWhatsOnSearch::seekTime = 0;
+int cMenuWhatsOnSearch::shiftTime = 0;
 
 cMenuWhatsOnSearch::cMenuWhatsOnSearch(const cSchedules *Schedules, int CurrentChannelNr)
    :cOsdMenu("", GetTab(1), GetTab(2), GetTab(3), GetTab(4), GetTab(5))
 {
-   shiftTime = 0;
-   schedules = Schedules;
-
-   // do a font patch, if necessary
-   if (strcasestr(cTemplFile::GetTemplateByName("MenuWhatsOnNow")->MenuTemplate(), "%progr%") ||
-       strcasestr(cTemplFile::GetTemplateByName("MenuWhatsOnNext")->MenuTemplate(), "%progr%"))
-   {
+  helpKeys = -1;
+  shiftTime = 0;
+  schedules = Schedules;
+  
+  // do a font patch, if necessary
+  if (strcasestr(cTemplFile::GetTemplateByName("MenuWhatsOnNow")->MenuTemplate(), "%progr%") ||
+      strcasestr(cTemplFile::GetTemplateByName("MenuWhatsOnNext")->MenuTemplate(), "%progr%"))
+    {
       if(Setup.UseSmallFont==2)  
-         PatchFont(fontSml);
+	PatchFont(fontSml);
       else
-         PatchFont(fontOsd);	
-   }
-
-   CreateShowModes();
-
-   LoadSchedules();
-
-   currentChannel = CurrentChannelNr;
-    
-   SetHelpKeys();
+	PatchFont(fontOsd);	
+    }
+  
+  CreateShowModes();
+  
+  LoadSchedules();
+  
+  currentChannel = CurrentChannelNr;
+  
+  SetHelpKeys();
 }
 
 cMenuWhatsOnSearch::~cMenuWhatsOnSearch()
 {
 }
 
+#ifdef USE_GRAPHTFT
+const char* cMenuWhatsOnSearch::MenuKind()
+{
+   if (currentShowMode == showNow)  return "MenuEpgsWhatsOnNow";
+   if (currentShowMode == showNext) return "MenuEpgsWhatsOnNext";
+   if (currentShowMode > showNext)  return "MenuEpgsWhatsOnElse";
+   else return "MenuWhatsOnElse";
+}
+#endif
+
 int cMenuWhatsOnSearch::GetTab(int Tab)
 {
    if (currentShowMode == showNow)
@@ -217,48 +315,47 @@ void cMenuWhatsOnSearch::LoadSchedules()
    Clear();
    eventObjects.Clear();
    
-   time_t SeekTime;
-   char* szTitle = NULL;
+   //   time_t SeekTime;
+   cString szTitle;
    cShowMode* mode = GetShowMode(currentShowMode);
 
    if (shiftTime != 0)
    {
       if (currentShowMode == showNow || currentShowMode == showNext)
-         SeekTime = time(NULL);
+         seekTime = time(NULL);
       else
       {
          if (mode)
-            SeekTime = GetTimeT(mode->GetTime());
-         if (SeekTime < time(NULL)) SeekTime += HOURS2SECS(24);	
+            seekTime = GetTimeT(mode->GetTime());
+         if (seekTime < time(NULL)) seekTime += HOURS2SECS(24);	
       }
-      SeekTime += shiftTime*60;
+      seekTime += shiftTime*60;
 	
       struct tm tm_r;
       time_t now = time(NULL);
 
-      tm tm_seek = *localtime_r(&SeekTime, &tm_r);
+      tm tm_seek = *localtime_r(&seekTime, &tm_r);
       tm tm_now = *localtime_r(&now, &tm_r);
       if (tm_seek.tm_mday != tm_now.tm_mday)
-         asprintf(&szTitle, "%s - %s", tr("Overview"), DAYDATETIME(SeekTime));
+	szTitle = cString::sprintf("%s - %s", tr("Overview"), DAYDATETIME(seekTime));
       else
-         asprintf(&szTitle, "%s - %02d:%02d", tr("Overview"), tm_seek.tm_hour, tm_seek.tm_min);
+	szTitle = cString::sprintf("%s - %02d:%02d", tr("Overview"), tm_seek.tm_hour, tm_seek.tm_min);
    }
    else
    {
       if (mode)
       {
-         SeekTime = GetTimeT(mode->GetTime());
-         if (SeekTime < time(NULL) && currentShowMode != showNow && currentShowMode != showNext)
+         seekTime = GetTimeT(mode->GetTime());
+         if (seekTime < time(NULL) && currentShowMode != showNow && currentShowMode != showNext)
          {
-            SeekTime += HOURS2SECS(24);	
-            asprintf(&szTitle, "%s - %s (%s)", tr("Overview"), mode->GetDescription(), *WeekDayName(SeekTime));    
+            seekTime += HOURS2SECS(24);	
+	    szTitle = cString::sprintf("%s - %s (%s)", tr("Overview"), mode->GetDescription(), *WeekDayName(seekTime));    
          }
          else
-            asprintf(&szTitle, "%s - %s", tr("Overview"), mode->GetDescription());    
+	   szTitle = cString::sprintf("%s - %s", tr("Overview"), mode->GetDescription());    
       }
    }
    SetTitle(szTitle);
-   free(szTitle);
 
    cMenuTemplate* currentTemplate = NULL;
    if (currentShowMode == showNow)
@@ -280,12 +377,12 @@ void cMenuWhatsOnSearch::LoadSchedules()
          if (EPGSearchConfig.showRadioChannels == 0 && ISRADIO(Channel))
             continue;
 
-         const cSchedule *Schedule = schedules->GetSchedule(Channel->GetChannelID());
+         const cSchedule *Schedule = schedules->GetSchedule(Channel);
          const cEvent *Event = NULL;
          if (Schedule) 
          {		
             if (shiftTime != 0)
-               Event = Schedule->GetEventAround(SeekTime);
+               Event = Schedule->GetEventAround(seekTime);
             else
             {
                switch(currentShowMode)
@@ -301,7 +398,7 @@ void cMenuWhatsOnSearch::LoadSchedules()
                   case showUserMode2:
                   case showUserMode3:
                   case showUserMode4:
-                     Event = Schedule->GetEventAround(SeekTime);
+                     Event = Schedule->GetEventAround(seekTime);
                      break;
                }
             }
@@ -314,12 +411,10 @@ void cMenuWhatsOnSearch::LoadSchedules()
       }
       else
       {
-         if (EPGSearchConfig.showChannelGroups)
+	if (EPGSearchConfig.showChannelGroups && strlen(Channel->Name()))
          {
-            char* szGroup = NULL;
-            asprintf(&szGroup, "----------------------------------------\t %s ----------------------------------------------------------------------------------------------", Channel->Name());
+	   cString szGroup = cString::sprintf("----------------------------------------\t %s ----------------------------------------------------------------------------------------------", Channel->Name());
             cOsdItem* pGroupItem = new cOsdItem(szGroup);
-            free(szGroup);
             pGroupItem->SetSelectable(false);
             Add(pGroupItem);
          }
@@ -356,9 +451,9 @@ void cMenuWhatsOnSearch::CreateShowModes()
 {
    showModes.Clear();
 
-   cShowMode* ModeNow = new cShowMode(showNow, tr("Button$Now"));
+   cShowMode* ModeNow = new cShowMode(showNow, trVDR("Button$Now"));
    showModes.Add(ModeNow);
-   cShowMode* ModeNext = new cShowMode(showNext, tr("Button$Next"));
+   cShowMode* ModeNext = new cShowMode(showNext, trVDR("Button$Next"));
    showModes.Add(ModeNext);
 
    time_t now = time(NULL);
@@ -407,7 +502,7 @@ eOSState cMenuWhatsOnSearch::Switch(void)
       if (cDevice::PrimaryDevice()->SwitchChannel(item->channel, true))
          return osEnd;
    }
-   Skins.Message(mtInfo, tr("Can't switch channel!")); 
+   Skins.Message(mtInfo, trVDR("Can't switch channel!")); 
    return osContinue;
 }
 
@@ -421,10 +516,12 @@ eOSState cMenuWhatsOnSearch::Record(void)
          int tm = tmNone;
          cTimer *timer = Timers.GetMatch(item->event, &tm);
          if (timer)
-            if (EPGSearchConfig.useVDRTimerEditMenu)
+	   {
+	     if (EPGSearchConfig.useVDRTimerEditMenu)
                return AddSubMenu(new cMenuEditTimer(timer));
-            else
+	     else
                return AddSubMenu(new cMenuMyEditTimer(timer, false, item->event, item->channel));
+	   }
       }
 
       cTimer *timer = NULL;
@@ -453,12 +550,12 @@ eOSState cMenuWhatsOnSearch::Record(void)
       else
       {
          string fullaux = "";
+         string aux = "";
          if (item->event)
          {
             const cEvent* event = item->event;
             int bstart = event->StartTime() - timer->StartTime();
             int bstop = timer->StopTime() - event->EndTime();
-            string aux = "";
             int checkmode = DefTimerCheckModes.GetMode(timer->Channel());
             aux = UpdateAuxValue(aux, "channel", NumToString(timer->Channel()->Number()) + " - " + CHANNELNAME(timer->Channel()));
             aux = UpdateAuxValue(aux, "update", checkmode);
@@ -467,8 +564,14 @@ eOSState cMenuWhatsOnSearch::Record(void)
             aux = UpdateAuxValue(aux, "bstop", bstop);
             fullaux = UpdateAuxValue(fullaux, "epgsearch", aux);
          }
+#ifdef USE_PINPLUGIN
+         aux = "";
+         aux = UpdateAuxValue(aux, "protected", timer->FskProtection() ? "yes" : "no");
+         fullaux = UpdateAuxValue(fullaux, "pin-plugin", aux);
+#endif
          SetAux(timer, fullaux);
          Timers.Add(timer);
+	 gl_timerStatusMonitor->SetConflictCheckAdvised(); 
          timer->Matches();
          Timers.SetModified();
          LogFile.iSysLog("timer %s added (active)", *timer->ToDescr());
@@ -524,26 +627,37 @@ eOSState cMenuWhatsOnSearch::ExtendedSearch(void)
    return AddSubMenu(new cMenuEPGSearchExt());
 }
 
-void cMenuWhatsOnSearch::SetHelpKeys()
+void cMenuWhatsOnSearch::SetHelpKeys(bool Force)
 {
-   cMenuMyScheduleItem *item = (cMenuMyScheduleItem *)Get(Current());
-   bool hasTimer = item && item->Selectable() && item->timerMatch == tmFull;
+  cMenuMyScheduleItem *item = (cMenuMyScheduleItem *)Get(Current());
+  int NewHelpKeys = 0;
+  if (item) {
+    if (item->Selectable() && item->timerMatch == tmFull)	
+      NewHelpKeys = 2;
+    else
+      NewHelpKeys = 1;
+  }
 
-   showMode nextShowMode = GetNextMode();    
-   cShowMode* mode = GetShowMode(nextShowMode);
-   char* szButtonGreen = NULL;
-   if (mode) 
-      szButtonGreen = mode->GetDescription();
-   if (toggleKeys==0)
-      SetHelp((EPGSearchConfig.redkeymode==0?(hasTimer?tr("Button$Timer"):tr("Button$Record")):tr("Button$Commands")), 
-              szButtonGreen, 
-              tr("Button$Schedule"), 
-              EPGSearchConfig.bluekeymode==0?(EPGSearchConfig.useOkForSwitch?tr("Button$Info"):tr("Button$Switch")):tr("Button$Search"));
-   else
-      SetHelp((EPGSearchConfig.redkeymode==1?(hasTimer?tr("Button$Timer"):tr("Button$Record")):tr("Button$Commands")), 
-              (EPGSearchConfig.toggleGreenYellow==0?szButtonGreen:"<<"), 
-              (EPGSearchConfig.toggleGreenYellow==0?tr("Button$Schedule"):">>"), 
-              EPGSearchConfig.bluekeymode==1?(EPGSearchConfig.useOkForSwitch?tr("Button$Info"):tr("Button$Switch")):tr("Button$Search"));
+  bool hasTimer = (NewHelpKeys == 2);
+  if (NewHelpKeys != helpKeys || Force)
+    { 
+      showMode nextShowMode = GetNextMode();    
+      cShowMode* mode = GetShowMode(nextShowMode);
+      char* szButtonGreen = NULL;
+      if (mode) 
+	szButtonGreen = mode->GetDescription();
+      if (toggleKeys==0)
+	SetHelp((EPGSearchConfig.redkeymode==0?(hasTimer?trVDR("Button$Timer"):trVDR("Button$Record")):tr("Button$Commands")), 
+		szButtonGreen, 
+		trVDR("Button$Schedule"), 
+		EPGSearchConfig.bluekeymode==0?(EPGSearchConfig.useOkForSwitch?trVDR("Button$Info"):trVDR("Button$Switch")):tr("Button$Search"));
+      else
+	SetHelp((EPGSearchConfig.redkeymode==1?(hasTimer?trVDR("Button$Timer"):trVDR("Button$Record")):tr("Button$Commands")), 
+		(EPGSearchConfig.toggleGreenYellow==0?szButtonGreen:"<<"), 
+		(EPGSearchConfig.toggleGreenYellow==0?trVDR("Button$Schedule"):">>"), 
+		EPGSearchConfig.bluekeymode==1?(EPGSearchConfig.useOkForSwitch?trVDR("Button$Info"):trVDR("Button$Switch")):tr("Button$Search"));
+      helpKeys = NewHelpKeys;
+    }
 }
 
 eOSState cMenuWhatsOnSearch::Shift(int iMinutes)
@@ -646,7 +760,7 @@ eOSState cMenuWhatsOnSearch::ProcessKey(eKeys Key)
             if(!HasSubMenu()) 
             {
                toggleKeys = 1 - toggleKeys;
-               SetHelpKeys();
+               SetHelpKeys(true);
             }
             state = osContinue;
             break;
@@ -659,7 +773,7 @@ eOSState cMenuWhatsOnSearch::ProcessKey(eKeys Key)
                   cMenuMyScheduleItem *mi = (cMenuMyScheduleItem *)Get(Current());
                   if (mi && mi->channel)
                   {
-                     const cSchedule *Schedule = schedules->GetSchedule(mi->channel->GetChannelID());
+                     const cSchedule *Schedule = schedules->GetSchedule(mi->channel);
                      if (Schedule)
                      {
                         time_t now = time(NULL);
@@ -722,7 +836,10 @@ eOSState cMenuWhatsOnSearch::ProcessKey(eKeys Key)
                return EPGSearchConfig.useOkForSwitch?Switch():ShowSummary();
          }
          break;
-         default:      break;
+      case kInfo:
+	return ShowSummary();
+	break;
+      default:      break;
       }
    }
    if (!HasSubMenu()) 
