@@ -1,3 +1,26 @@
+/*
+Copyright (C) 2004-2008 Christian Wieninger
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 2
+of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+Or, point your browser to http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+
+The author can be reached at cwieninger@gmx.de
+
+The project's page is at http://winni.vdr-developer.org/epgsearch
+*/
+
 #include <vector>
 #include "searchtimer_thread.h"
 #include "epgsearchcfg.h"
@@ -16,6 +39,8 @@
 #include "timer_thread.h"
 #include "timerdone.h"
 #include "menu_deftimercheckmethod.h"
+#include "timerstatus.h"
+#include "epgsearch.h"
 
 // priority for background thread
 #define SEARCHTIMER_NICE 19
@@ -81,6 +106,9 @@ cTimer *cSearchTimerThread::GetTimer(cSearchExt *searchExt, const cEvent *pEvent
    bool UseVPS = searchExt->useVPS && pEvent->Vps() && Setup.UseVps;
    time_t eStart = pEvent->StartTime();
    time_t eStop = pEvent->EndTime();
+   int eDuration = pEvent->Duration();
+
+   int AllowedDiff = (eDuration < 10*60) ? (eDuration / 60): 10; // allowed start/stop difference
 
    int testVpsStart = 0;
    int testVpsStop = 0;
@@ -115,8 +143,8 @@ cTimer *cSearchTimerThread::GetTimer(cSearchExt *searchExt, const cEvent *pEvent
          int timerVpsStart = cTimer::TimeToInt(ti->Start());
          int timerVpsStop = cTimer::TimeToInt(ti->Stop());
 
-         if (abs(testVpsStart - timerVpsStart) > 10 * 60) continue;
-         if (abs(testVpsStop - timerVpsStop) > 10 * 60) continue;
+         if (abs(testVpsStart - timerVpsStart) > AllowedDiff * 60) continue;
+         if (abs(testVpsStop - timerVpsStop) > AllowedDiff * 60) continue;
 
          bTimesMatchExactly = (testVpsStart == timerVpsStart && testVpsStop == timerVpsStop);
          return ti;
@@ -131,7 +159,7 @@ cTimer *cSearchTimerThread::GetTimer(cSearchExt *searchExt, const cEvent *pEvent
 	    
          bTimesMatchExactly = (tStart == eStart && tStop == eStop);
 	
-         if (abs(tStart - eStart) < 10 * 60 && abs(tStop - eStop) < 10 * 60) // accept a difference of max 10 min.
+         if (abs(tStart - eStart) < AllowedDiff * 60 && abs(tStop - eStop) < AllowedDiff * 60) // accept a difference of max 10 min., but only if the event duration is more than 10 minutes
             return ti;
       }
    }
@@ -195,9 +223,9 @@ void cSearchTimerThread::Action(void)
          cSearchExt *searchExt = localSearchExts->First();
          // reset announcelist
          announceList.Clear();
-         while (searchExt) 
+         while (searchExt && m_Active) 
          {
-            if (!searchExt->useAsSearchTimer)
+	   if (!searchExt->IsActiveAt(now))
             {
                searchExt = localSearchExts->Next(searchExt);
                continue;
@@ -246,6 +274,8 @@ void cSearchTimerThread::Action(void)
                cTimer *t = GetTimer(searchExt, pEvent, bTimesMatchExactly);
 
                char* Summary = NULL;
+	       uint timerMod = tmNoChange;
+
                if (t) 
                { // already exists
                   pOutdatedTimers->DelTimer(t);
@@ -286,7 +316,7 @@ void cSearchTimerThread::Action(void)
                   }
 
                   char* pFile = NULL; // File is prepared for svdrp, so prepare t->File for comparision too
-                  asprintf(&pFile, "%s", t->File());
+                  msprintf(&pFile, "%s", t->File());
                   pFile = strreplace(pFile, ':', '|');
                   pFile = strreplace(pFile, " ~", "~");
                   pFile = strreplace(pFile, "~ ", "~");
@@ -304,18 +334,30 @@ void cSearchTimerThread::Action(void)
                   }
                   else
                   {
-                     if (LogFile.Level() >= 3) // output reasons for a timer modification 
-                     { 
-                        if (!bTimesMatchExactly)
-                           LogFile.Log(3,"timer for '%s~%s' (%s - %s, channel %d) : start/stop has changed", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent));
-                        if (strcmp(pFile, timer->File()) != 0)
-                           LogFile.Log(3,"timer for '%s~%s' (%s - %s, channel %d) : title and/or episdode has changed", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent));
-                        if (t->Aux() != NULL && strcmp(t->Aux(), Summary) != 0)
-                           LogFile.Log(3,"timer for '%s~%s' (%s - %s, channel %d) : aux info has changed", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent));
-                     }
-                     index = t->Index()+1;
-                     Priority = t->Priority();
-                     Lifetime = t->Lifetime();
+		    if (!bTimesMatchExactly) timerMod = (uint)timerMod | tmStartStop;
+		    if (strcmp(pFile, timer->File()) != 0) timerMod |= tmFile;
+		    if (t->Aux() != NULL && strcmp(t->Aux(), Summary) != 0)
+		      {
+			char* oldEventID = GetAuxValue(t, "eventid");
+			char* newEventID = GetAuxValue(Summary, "eventid");
+			if (oldEventID && newEventID && strcmp(oldEventID, newEventID) != 0)
+			  timerMod |= tmAuxEventID;
+			free(oldEventID);
+			free(newEventID);
+		      }
+
+		    if (LogFile.Level() >= 3) // output reasons for a timer modification 
+		      { 
+                        if (timerMod & tmStartStop)
+			  LogFile.Log(3,"timer for '%s~%s' (%s - %s, channel %d) : start/stop has changed", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent));
+                        if (timerMod & tmFile)
+			  LogFile.Log(3,"timer for '%s~%s' (%s - %s, channel %d) : title and/or episdode has changed", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent));
+                        if (timerMod & tmAuxEventID)
+			  LogFile.Log(3,"timer for '%s~%s' (%s - %s, channel %d) : aux info for event id has changed", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent));
+		      }
+		    index = t->Index()+1;
+		    Priority = t->Priority();
+		    Lifetime = t->Lifetime();
                   }
                   free(pFile);
 
@@ -340,7 +382,7 @@ void cSearchTimerThread::Action(void)
                {
                   if (t || // timer already exists or
                       NoAnnounces.InList(pEvent) || // announcement not wanted anymore or
-                      (EPGSearchConfig.noAnnounceWhileReplay && cDevice::PrimaryDevice()->Replaying()) // no announce while replay
+                      (EPGSearchConfig.noAnnounceWhileReplay && cDevice::PrimaryDevice()->Replaying() && updateForced != 2)  // no announce while replay within automatic updates
                      ) 
                   {
                      if (Summary) free(Summary);
@@ -362,7 +404,8 @@ void cSearchTimerThread::Action(void)
                      if (!SwitchTimers.InSwitchList(pEvent))
                      {			
                         cMutexLock SwitchTimersLock(&SwitchTimers);	
-                        SwitchTimers.Add(new cSwitchTimer(pEvent, searchExt->switchMinsBefore));
+                        SwitchTimers.Add(new cSwitchTimer(pEvent, searchExt->switchMinsBefore, 0, 
+							  searchExt->unmuteSoundOnSwitch));
                         SwitchTimers.Save();
                         cSwitchTimerThread::Init(); 
                      }
@@ -372,7 +415,7 @@ void cSearchTimerThread::Action(void)
                   continue;
                }
 			
-               if (AddModTimer(timer, index, searchExt, pEvent, Priority, Lifetime, Summary))
+               if (AddModTimer(timer, index, searchExt, pEvent, Priority, Lifetime, Summary, timerMod))
                {
                   if (index == 0)
                      LogFile.Log(1,"added timer for '%s~%s' (%s - %s); search timer: '%s'", pEvent->Title(), pEvent->ShortText()?pEvent->ShortText():"", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), searchExt->search);
@@ -420,14 +463,12 @@ void cSearchTimerThread::Action(void)
 
          if (announceList.Count() > 0)
          {
-            char* msgfmt = NULL;
-            asprintf(&msgfmt, tr("%d new broadcast(s) found! Show them?"), announceList.Count());
-            if (SendMsg(msgfmt, true,7) == kOk)		   
-            {
-                  m_plugin->showAnnounces = true;
-                  cRemote::CallPlugin("epgsearch");	
-            }
-            free(msgfmt);
+	   cString msgfmt = cString::sprintf(tr("%d new broadcast(s) found! Show them?"), announceList.Count());
+	   if (SendMsg(msgfmt, true,7) == kOk)		   
+	     {
+	       m_plugin->showAnnounces = true;
+	       cRemote::CallPlugin("epgsearch");	
+	     }
          }
 
          LogFile.iSysLog("search timer update finished");
@@ -447,18 +488,18 @@ void cSearchTimerThread::Action(void)
                   mailNotifier.SendConflictNotifications(conflictCheck);
                }
 
-               char* msgfmt = NULL;
-               asprintf(&msgfmt, tr("%d timer conflict(s)! First at %s. Show them?"), conflictCheck.relevantConflicts,
-                        *DateTime(conflictCheck.nextRelevantConflictDate));
+               cString msgfmt = cString::sprintf(tr("%d timer conflict(s)! First at %s. Show them?"), 
+						 conflictCheck.relevantConflicts,
+						 *DateTime(conflictCheck.nextRelevantConflictDate));
                bool doMessage = EPGSearchConfig.noConflMsgWhileReplay == 0 || 
                   !cDevice::PrimaryDevice()->Replaying() || 
-                  conflictCheck.nextRelevantConflictDate - now < 2*60*60;
+                  conflictCheck.nextRelevantConflictDate - now < 2*60*60 ||
+		 updateForced == 2;
                if (doMessage && SendMsg(msgfmt, true,7) == kOk)			
                {
                   m_plugin->showConflicts = true;
                   cRemote::CallPlugin("epgsearch");	
                }
-               free(msgfmt);
             }
 		
             LogFile.iSysLog("check for timer conflicts - done");	    
@@ -510,7 +551,7 @@ char* cSearchTimerThread::SummaryExtended(cSearchExt* searchExt, cTimer* Timer,
    time_t stop  = eStop + (UseVPS?0:(searchExt->MarginStop * 60));
 
    char* addSummaryFooter = NULL;
-   asprintf(&addSummaryFooter, "<channel>%d - %s</channel><searchtimer>%s</searchtimer><start>%ld</start><stop>%ld</stop><s-id>%d</s-id><eventid>%ld</eventid>", 
+   msprintf(&addSummaryFooter, "<channel>%d - %s</channel><searchtimer>%s</searchtimer><start>%ld</start><stop>%ld</stop><s-id>%d</s-id><eventid>%ld</eventid>", 
             Timer->Channel()->Number(), CHANNELNAME(Timer->Channel()),
             searchExt->search, 
             start, 
@@ -534,13 +575,13 @@ char* cSearchTimerThread::SummaryExtended(cSearchExt* searchExt, cTimer* Timer,
    }
 
    char* tmpSummary = NULL;
-   asprintf(&tmpSummary, "<epgsearch>%s</epgsearch>%s", addSummaryFooter, tmpaux?tmpaux:"");
+   msprintf(&tmpSummary, "<epgsearch>%s</epgsearch>%s", addSummaryFooter, tmpaux?tmpaux:"");
    free(addSummaryFooter);
    if (tmpaux) free(tmpaux);
    return tmpSummary;
 }
 
-bool cSearchTimerThread::AddModTimer(cTimer* Timer, int index, cSearchExt* searchExt, const cEvent* pEvent, int Prio, int Lifetime, char* Summary) 
+bool cSearchTimerThread::AddModTimer(cTimer* Timer, int index, cSearchExt* searchExt, const cEvent* pEvent, int Prio, int Lifetime, char* Summary, uint timerMod) 
 {
    char *cmdbuf = NULL;
 
@@ -581,10 +622,14 @@ bool cSearchTimerThread::AddModTimer(cTimer* Timer, int index, cSearchExt* searc
       tmpSummary = SummaryExtended(searchExt, Timer, pEvent);
 
    if (index==0)
-      asprintf(&cmdbuf, "NEWT %d:%d:%s:%s:%s:%d:%d:%s:%s", 
+      msprintf(&cmdbuf, "NEWT %d:%d:%s:%s:%s:%d:%d:%s:%s", 
                Flags,
                Timer->Channel()->Number(),
+#if VDRVERSNUM < 10503
                *Timer->PrintDay(start, Timer->WeekDays()),
+#else
+               *Timer->PrintDay(start, Timer->WeekDays(), true),
+#endif
                bufStart,
                bufEnd,
                Prio, 
@@ -592,11 +637,15 @@ bool cSearchTimerThread::AddModTimer(cTimer* Timer, int index, cSearchExt* searc
                Timer->File(),
                tmpSummary?tmpSummary:"");
    else
-      asprintf(&cmdbuf, "MODT %d %d:%d:%s:%s:%s:%d:%d:%s:%s", 
+      msprintf(&cmdbuf, "MODT %d %d:%d:%s:%s:%s:%d:%d:%s:%s", 
                index,
                Flags,
                Timer->Channel()->Number(),
+#if VDRVERSNUM < 10503
                *Timer->PrintDay(start, Timer->WeekDays()),
+#else
+               *Timer->PrintDay(start, Timer->WeekDays(), true),
+#endif
                bufStart,
                bufEnd,
                Prio, 
@@ -605,6 +654,7 @@ bool cSearchTimerThread::AddModTimer(cTimer* Timer, int index, cSearchExt* searc
                tmpSummary?tmpSummary:"");
 
    SendViaSVDRP(cmdbuf);
+   if (gl_timerStatusMonitor) gl_timerStatusMonitor->SetConflictCheckAdvised(); 
 
    cTimerDone* timerdone = new cTimerDone(start, stop, pEvent, searchExt->ID);
    if (index==0)
@@ -617,7 +667,7 @@ bool cSearchTimerThread::AddModTimer(cTimer* Timer, int index, cSearchExt* searc
       if (index==0) // new
          mailNotifier.AddNewTimerNotification(pEvent->EventID(), pEvent->ChannelID()); 
       else
-         mailNotifier.AddModTimerNotification(pEvent->EventID(), pEvent->ChannelID()); 
+	mailNotifier.AddModTimerNotification(pEvent->EventID(), pEvent->ChannelID(), timerMod); 
    }
    free(cmdbuf);
    if (tmpSummary) free(tmpSummary);
@@ -645,12 +695,9 @@ void cSearchTimerThread::RemoveTimer(cTimer* t, const cEvent* e)
 
 void cSearchTimerThread::DelRecording(int index)
 {
-   char *cmdbuf = NULL;   // first LSTR required 
-   asprintf(&cmdbuf, "DELR %d", index);    
-   LogFile.Log(2, "delete recording %d", index);
-   SendViaSVDRP(cmdbuf);
-	      
-   free(cmdbuf); 
+  cString cmdbuf = cString::sprintf("DELR %d", index);    
+  LogFile.Log(2, "delete recording %d", index);
+  SendViaSVDRP(cmdbuf);
 }
 
 void cSearchTimerThread::CheckExpiredRecs()
@@ -738,7 +785,7 @@ void cSearchTimerThread::ModifyManualTimer(const cEvent* event, const cTimer* ti
    strftime(stopbuffer, DAYBUFFERSIZE, "%H%M", &tm_r_stop);
     
    char* cmdbuf = NULL;
-   asprintf(&cmdbuf, "MODT %d %d:%d:%s:%s:%s:%d:%d:%s:%s", 
+   msprintf(&cmdbuf, "MODT %d %d:%d:%s:%s:%s:%d:%d:%s:%s", 
             timer->Index()+1,
             timer->Flags(),
             timer->Channel()->Number(),
@@ -836,8 +883,10 @@ void cSearchTimerThread::CheckManualTimers()
                   }
                }
                LogFile.Log(3,"selected candidate is '%s~%s' (%s - %s)", event->Title(), event->ShortText()?event->ShortText():"", GETDATESTRING(event), GETTIMESTRING(event));
-               if (event->StartTime() - bstart != ti->StartTime() || event->EndTime() + bstop != ti->StopTime())
+               if ((maxweight > 0 && event->StartTime() - bstart != ti->StartTime()) || (event->EndTime() + bstop != ti->StopTime()))
                   ModifyManualTimer(event, ti, bstart, bstop);		    
+	       else if (maxweight == 0)
+		 LogFile.Log(3,"selected candidate is too bad");
             }
             else
                LogFile.Log(1,"ooops - no events found touching manual timer %d", ti->Index()+1);
