@@ -1,3 +1,26 @@
+/*
+Copyright (C) 2004-2008 Christian Wieninger
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 2
+of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+Or, point your browser to http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+
+The author can be reached at cwieninger@gmx.de
+
+The project's page is at http://winni.vdr-developer.org/epgsearch
+*/
+
 #ifndef __USERVARS_INC__
 #define __USERVARS_INC__
 
@@ -134,7 +157,15 @@ public:
 class cEventIDVar : public cInternalVar {
 public:
     cEventIDVar() : cInternalVar("%eventid%") {}
-    string Evaluate(const cEvent* e,  bool escapeStrings = false) { if (e) return NumToString((long)e->EventID()); else return ""; }
+    string Evaluate(const cEvent* e,  bool escapeStrings = false) 
+    { 
+      if (e) 
+	{
+	  ostringstream os;
+	  os << e->EventID();
+	  return os.str();
+	}
+      else return ""; }
 };
 
 class cTimeVar : public cInternalVar {
@@ -204,33 +235,30 @@ public:
 	    string res;
 	    if (labs(diff) >= SECSINDAY)
 	    {
-		char* buffer = NULL;
-		if (diff > 0)
-		    asprintf(&buffer, tr("in %02ldd"), long(diff / SECSINDAY));
-		else
-		    asprintf(&buffer, "%02ldd", long(-diff / SECSINDAY));
-		res = buffer;
-		free(buffer);
+	      cString buffer;
+	      if (diff > 0)
+		buffer = cString::sprintf(tr("in %02ldd"), long(diff / SECSINDAY));
+	      else
+		buffer = cString::sprintf("%02ldd", long(-diff / SECSINDAY));
+	      res = buffer;
 	    }
 	    else if (labs(diff) >= (60 * 60))
 	    {
-		char* buffer = NULL;
-		if (diff > 0)
-		    asprintf(&buffer, tr("in %02ldh"), long(diff / (60*60)));
-		else
-		    asprintf(&buffer, "%02ldh", long(-diff / (60*60)));
-		res = buffer;
-		free(buffer);
+	      cString buffer;
+	      if (diff > 0)
+		buffer = cString::sprintf(tr("in %02ldh"), long(diff / (60*60)));
+	      else
+		buffer = cString::sprintf("%02ldh", long(-diff / (60*60)));
+	      res = buffer;
 	    }
 	    else
 	    {
-		char* buffer = NULL;
-		if (diff > 0)
-		    asprintf(&buffer, tr("in %02ldm"), long(diff / 60));
-		else
-		    asprintf(&buffer, "%02ldm", long(-diff / 60));
-		res = buffer;
-		free(buffer);
+	      cString buffer;
+	      if (diff > 0)
+		buffer = cString::sprintf(tr("in %02ldm"), long(diff / 60));
+	      else
+		buffer = cString::sprintf("%02ldm", long(-diff / 60));
+	      res = buffer;
 	    }
 	    if (escapeStrings) return "'" + EscapeString(res) + "'"; else return res;
 	}
@@ -642,14 +670,14 @@ class cUserVars : public cList<cUserVar> {
 		delete (*uvar);
 	    userVars.clear();
 	}
-    cUserVar* GetFromName(const string& varName);
+    cUserVar* GetFromName(const string& varName, bool log = true);
 };
 
 extern cUserVars UserVars;
 
 class cUserVarLine : public cListObject {
 public:
-  bool Parse(char *s);
+  static bool Parse(char *s);
 };
 
 
